@@ -1,5 +1,6 @@
 #' Calculates the Lobo Index (LOBO)
 #' @param resultLoad The resulting list obtained from the diat_loadData() function
+#' @param maxDistTaxa Integer. Number of characters that can differ in the species' names when compared to the internal database's name in the heuristic search. Default = 2
 #' @description
 #' The input for all of these functions is the resulting dataframe (resultLoad) obtained from the diat_loadData() function
 #' A CSV or dataframe cannot be used directly with these functions, they have to be loaded first with the diat_loadData() function
@@ -14,7 +15,7 @@
 #'
 #' Sample data in the examples is taken from:
 #' \itemize{
-#' \item Nicolosi Gelis, María Mercedes; Cochero, Joaquín; Donadelli, Jorge; Gómez, Nora. 2020. "Exploring the use of nuclear alterations, motility and ecological guilds in epipelic diatoms as biomonitoring tools for water quality improvement in urban impacted lowland streams". Ecological Indicators, 110, 105951. https://doi.org/10.1016/j.ecolind.2019.105951
+#' \item Nicolosi Gelis, María Mercedes; Cochero, Joaquín; Donadelli, Jorge; Gómez, Nora. 2020. "Exploring the use of nuclear alterations, motility and ecological guilds in epipelic diatoms as biomonitoring tools for water quality improvement in urban impacted lowland streams". Ecological Indicators, 110, 105951. https://doi:10.1016/j.ecolind.2019.105951
 #' }
 #' @examples
 #' \donttest{
@@ -34,7 +35,7 @@
 ###### ---------- FUNCTION FOR LOBO INDEX (Lobo et al. 2002)---------- ########
 ### INPUT: resultLoad Data cannot be in Relative Abuncance
 ### OUTPUTS: dataframe with LOBO index per sample
-diat_lobo <- function(resultLoad){
+diat_lobo <- function(resultLoad, maxDistTaxa = 2){
 
   # First checks if species data frames exist. If not, loads them from CSV files
   if(missing(resultLoad)) {
@@ -47,10 +48,7 @@ diat_lobo <- function(resultLoad){
 
   taxaIn <- resultLoad[[2]]
 
-
-  ### START NEW CORRECTIONS
   #Loads the species list specific for this index
-  #loboDB <- read.csv("../Indices/lobo.csv") #uses the external csv file
   loboDB <- diathor::lobo
 
 
@@ -58,21 +56,62 @@ diat_lobo <- function(resultLoad){
   taxaIn$species <- row.names(taxaIn)
 
 
-  # #exact matches species in input data to acronym from index
-  # taxaIn$lobo_v <- loboDB$lobo_v[match(taxaIn$acronym, trimws(loboDB$acronym))]
-  # taxaIn$lobo_s <- loboDB$lobo_s[match(taxaIn$acronym, trimws(loboDB$acronym))]
-
   # #the ones still not found (NA), try against fullspecies
   taxaIn$lobo_v <- NA
   taxaIn$lobo_s <- NA
+  print("Calculating LOBO index")
   for (i in 1:nrow(taxaIn)) {
     if (is.na(taxaIn$lobo_s[i]) | is.na(taxaIn$lobo_v[i])){
-      taxaIn$lobo_v[i] <- loboDB$lobo_v[match(trimws(rownames(taxaIn[i,])), trimws(loboDB$fullspecies))]
-      taxaIn$lobo_s[i] <- loboDB$lobo_s[match(trimws(rownames(taxaIn[i,])), trimws(loboDB$fullspecies))]
+      # New in v0.0.8
+      # Uses the stringdist package to find species by names heuristically, with a maximum distance = maxDistTaxa
+      # if multiple are found, uses majority consensus to select the correct index value
+      # 1) find the species by heuristic search
+      spname <- trimws(tolower(rownames(taxaIn[i,])))
+
+      species_found <- loboDB[stringdist::ain(trimws(tolower(loboDB$fullspecies)),spname, maxDist=maxDistTaxa, matchNA = FALSE),]
+      # 2) if found, build majority consensus for sensitivity values
+      if (nrow(species_found) == 1){
+        vvalue <- as.numeric(names(which.max(table(species_found$lobo_v))))
+        svalue <- as.numeric(names(which.max(table(species_found$lobo_s))))
+        taxaIn$new_species[i] <- species_found$fullspecies[1]
+      } else if (nrow(species_found) > 1){
+        species_found <- species_found[match(spname, trimws(tolower(species_found$fullspecies)), nomatch=1),]
+        vvalue <- as.numeric(names(which.max(table(species_found$lobo_v))))
+        svalue <- as.numeric(names(which.max(table(species_found$lobo_s))))
+      } else if (nrow(species_found) == 0){
+        #species not found, try tautonomy in variety
+        spsplit <- strsplit(spname, " ") #split the name
+        #if has epiteth
+        if (length(spsplit[[1]])>1){
+          #create vectors with possible epiteths
+          newspname <- paste(spsplit[[1]][[1]], spsplit[[1]][[2]], "var.", spsplit[[1]][[length(spsplit[[1]])]], sep = " ") #create new sp name
+          newspname <- c(newspname, paste(spsplit[[1]][[1]], spsplit[[1]][[2]], "fo.", spsplit[[1]][[length(spsplit[[1]])]], sep = " ")) #create new sp name
+          newspname <- c(newspname, paste(spsplit[[1]][[1]], spsplit[[1]][[2]], "subsp.", spsplit[[1]][[length(spsplit[[1]])]], sep = " ")) #create new sp name
+          newspname <- c(newspname, paste(spsplit[[1]][[1]], spsplit[[1]][[2]], "spp.", spsplit[[1]][[length(spsplit[[1]])]], sep = " ")) #create new sp name
+          newspname <- c(newspname, paste(spsplit[[1]][[1]], spsplit[[1]][[2]], "ssp.", spsplit[[1]][[length(spsplit[[1]])]], sep = " ")) #create new sp name
+          newspname <- c(newspname, paste(spsplit[[1]][[1]], spsplit[[1]][[2]], "var.", spsplit[[1]][[2]], "fo.", spsplit[[1]][[length(spsplit[[1]])]], sep = " ")) #create new sp name
+
+          #search again against all possible epiteths
+          species_found <- loboDB[stringdist::ain(trimws(tolower(loboDB$fullspecies)),newspname, maxDist=maxDistTaxa, matchNA = FALSE),]
+          if (nrow(species_found) > 0){
+            #found with tautonomy
+            vvalue <- as.numeric(names(which.max(table(species_found$lobo_v[1]))))
+            svalue <- as.numeric(names(which.max(table(species_found$lobo_s[1]))))
+            taxaIn$new_species[i] <- species_found$fullspecies[1]
+          } else {
+            #species not found, make everything NA
+            vvalue = NA
+            svalue = NA
+          }
+        }
+      }
+
+      #records the final consensus value
+      taxaIn$lobo_v[i] <- vvalue
+      taxaIn$lobo_s[i] <- svalue
     }
   }
 
-
   #removes NA from taxaIn
   taxaIn[is.na(taxaIn)] <- 0
 
@@ -80,40 +119,46 @@ diat_lobo <- function(resultLoad){
   lastcol <- which(colnames(taxaIn)=="new_species")
 
   #######--------LOBO INDEX START --------#############
-  print("Calculating LOBO index")
+
   #creates results dataframe
   lobo.results <- data.frame(matrix(ncol = 3, nrow = (lastcol-1)))
-  colnames(lobo.results) <- c("LOBO", "LOBO20", "Precision")
+  colnames(lobo.results) <- c("LOBO", "LOBO20", "num_taxa")
   #finds the column
   lobo_s <- (taxaIn[,"lobo_s"])
   lobo_v <- (taxaIn[,"lobo_v"])
-  #PROGRESS BAR
+
+  # Prints the number of taxa recognized for this index, regardless of their abundance
+  # It is therefore the same for all samples
+
+  number_recognized_taxa <- round((100 - (sum(is.na(taxaIn$lobo_s)) / nrow(taxaIn))*100),1)
+  print(paste("Taxa recognized to be used in LOBO index: ", number_recognized_taxa, "%"))
+
+    #PROGRESS BAR
   pb <- txtProgressBar(min = 1, max = (lastcol-1), style = 3)
   for (sampleNumber in 1:(lastcol-1)){ #for each sample in the matrix
-    #how many taxa will be used to calculate?
-    LOBOtaxaused <- (length(lobo_s) - sum(is.na(taxaIn$lobo_s )))*100 / length(lobo_s)
-    LOBOtaxaused <- (length(which(lobo_s * taxaIn[,sampleNumber] > 0))*100 / length(lobo_s))
+    #Revised v0.0.8
+    num_taxa <- length(which(lobo_s * taxaIn[,sampleNumber] > 0))
     #remove the NA
     lobo_s[is.na(lobo_s)] = 0
     lobo_v[is.na(lobo_v)] = 0
     LOBO <- sum((taxaIn[,sampleNumber]*as.double(lobo_s)*as.double(lobo_v)))/sum(taxaIn[,sampleNumber]*as.double(lobo_v)) #raw value
     LOBO20 <- (6.333*LOBO)-5.333
-    lobo.results[sampleNumber, ] <- c(LOBO, LOBO20,LOBOtaxaused)
+    lobo.results[sampleNumber, ] <- c(LOBO, LOBO20,num_taxa)
     #update progressbar
     setTxtProgressBar(pb, sampleNumber)
   }
   #close progressbar
   close(pb)
   #######--------LOBO INDEX: END--------############
-  #PRECISION
+  #PRECISION RECORDING
   resultsPath <- resultLoad[[4]]
-  #precisionmatrix <- read.csv(paste(resultsPath,"\\Precision.csv", sep=""))
-  precisionmatrix <- read.csv(file.path(resultsPath, "Precision.csv"))
-  precisionmatrix <- cbind(precisionmatrix, lobo.results$Precision)
+  #reads the csv file
+  precisionmatrix <- read.csv(file.path(resultsPath, "num_taxa.csv"))
+  #joins with the precision column
+  precisionmatrix <- cbind(precisionmatrix, lobo.results$num_taxa)
   precisionmatrix <- precisionmatrix[-(1:which(colnames(precisionmatrix)=="Sample")-1)]
-  names(precisionmatrix)[names(precisionmatrix)=="lobo.results$Precision"] <- "LOBO"
-  #write.csv(precisionmatrix, paste(resultsPath,"\\Precision.csv", sep=""))
-  write.csv(precisionmatrix, file.path(resultsPath, "Precision.csv"))
+  names(precisionmatrix)[names(precisionmatrix)=="lobo.results$num_taxa"] <- "LOBO"
+  write.csv(precisionmatrix, file.path(resultsPath, "num_taxa.csv"))
   #END PRECISION
 
   #TAXA INCLUSION
